#!/bin/bash

GRUBCONF=/boot/grub2/grub.cfg
KLOGFILE=/tmp/kernelCheck.log

now=`date`
echo "kernel check at $now" >> $KLOGFILE
kernels=(`rpm -qa --last | grep ^kernel-[1-9] | awk -F'-' '/^kernel-[1-9]/{printf "%s-%s\n",$2,$3}' | cut -d " " -f 1`)

num=${#kernels[*]}
echo "$num kernel rpms found" >> $KLOGFILE

if [ $num -eq 2 ]; then
    echo "Old kernel version is ${kernels[1]}" >> $KLOGFILE
    echo "New kernel version is ${kernels[0]}" >> $KLOGFILE
    echo "Update grub" >> $KLOGFILE
    sed -i "/^default=1/d" $GRUBCONF
    sed -i "/CentOS/d" $GRUBCONF
    sed -i "/${kernels[0]}/d" $GRUBCONF
    sed -i "s~${kernels[1]}~${kernels[0]}~g" $GRUBCONF
    echo "Remove old kernel" >> $KLOGFILE
    rpm -ev --nodeps kernel-${kernels[1]} &>> $KLOGFILE
    rpm_retval=$?
    if [ $rpm_retval -ne 0 ]; then
        echo "Removing old kernel failed with errorcode: $rpm_retval" >> $KLOGFILE
        exit $rpm_retval
    fi
    now=`date`
    echo "kernel sanitized at $now" >> $KLOGFILE
    exit 100
else
    echo "Kernels:" >> $KLOGFILE
    for kernel in ${kernels[@]} ; do
        echo $kernel >> $KLOGFILE
    done
    running_kernel=`uname -r`
    echo "Loaded kernel: $running_kernel" >> $KLOGFILE
    now=`date`
    echo "kernel verified at $now" >> $KLOGFILE
fi
exit 0
