#!/bin/bash

PLOGFILE=/tmp/postgreCheck.log

now=`date`
echo "postgre check at $now" >> $PLOGFILE
postgres=(`rpm -qa --last | grep ^postgresql[1-9].-[1-9]. | awk -F'-' '{printf "%s-%s\n",$2,$1}'`)

num=${#postgres[*]}
echo "$num postgre rpms found" >> $PLOGFILE

if [ $num -eq 2 ]; then
    
    echo "Old postgres version is ${postgres[1]}" >> $PLOGFILE
    echo "New postgres version is ${postgres[0]}" >> $PLOGFILE

    #Derive new paths
    new_postgres_package=$( echo "${postgres[0]}" | awk -F'-' '{printf "%s\n",$2}' )
    new_postgres_version=$( echo "${postgres[0]}" | sed 's|\(.*\)\..*|\1|' )
    new_pgsql_data_path=/var/lib/pgsql/${new_postgres_version}/data
    new_pgsql_bin_path=/usr/pgsql-${new_postgres_version}/bin
    new_postgres_process=postgresql-${new_postgres_version}

    #Derive old paths
    old_postgres_package=$( echo "${postgres[1]}" | awk -F'-' '{printf "%s\n",$2}' )
    old_postgres_version=$( echo "${postgres[1]}" | sed 's|\(.*\)\..*|\1|' )
    old_pgsql_data_path=/var/lib/pgsql/${old_postgres_version}/data
    old_pgsql_bin_path=/usr/pgsql-${old_postgres_version}/bin
    old_postgres_process=postgresql-${old_postgres_version}

    #Before running pg_upgrade you must:
    #  create a new database cluster (using the new version of initdb)
    if [ ! -e ${new_pgsql_data_path}/PG_VERSION ]; then 
        echo "--------- PG_VERSION does not exist, run PostgreSQL 11 initdb ---------" >> $PLOGFILE
        rm -rf ${new_pgsql_data_path}/*
        ${new_pgsql_bin_path}/postgresql-11-setup initdb >> $PLOGFILE 2>&1
        systemctl enable ${new_postgres_process}
    fi
    #  shutdown the postmaster servicing the old cluster
    systemctl stop ${old_postgres_process}
    #  shutdown the postmaster servicing the new cluster
    systemctl stop ${new_postgres_process}

    echo "--------- Upgrading database ---------" >> $PLOGFILE
    su - postgres -c "${new_pgsql_bin_path}/pg_upgrade -U postgres -v -d ${old_pgsql_data_path} -D ${new_pgsql_data_path} -b ${old_pgsql_bin_path} -B ${new_pgsql_bin_path}" >> ${PLOGFILE} 2>&1
    pgu_retval=$?
    if [ $pgu_retval -ne 0 ]; then
        echo "Upgrading old postgres failed with errorcode: $pgu_retval"
        exit $pgu_retval
    fi
    systemctl start ${new_postgres_process}
    echo "------- Analysing the new cluster -------" >> $PLOGFILE
    /var/lib/pgsql/analyze_new_cluster.sh >> ${PLOGFILE} 2>&1
    echo "------- Deleting the old cluster -------" >> $PLOGFILE
    /var/lib/pgsql/delete_old_cluster.sh >> ${PLOGFILE} 2>&1

    echo "--------- Uninstalling old ---------" >> $PLOGFILE
    yum --assumeyes autoremove ${old_postgres_package} >> $PLOGFILE 2>&1
    yum_retval=$?
    if [ $yum_retval -ne 0 ]; then
        echo "Removing old postgres failed with errorcode: $yum_retval"
        exit $yum_retval
    else
        rm -rf /var/lib/pgsql/${old_postgres_version}
        rm -rf /usr/pgsql-${old_postgres_version}
    fi
    now=`date`
    echo "postgres sanitized at $now" >> $PLOGFILE
else
    echo "Postgres:" >> $PLOGFILE
    for postgre in ${postgres[@]} ; do
        echo $postgre >> $PLOGFILE
    done
    now=`date`
    echo "postgres verified at $now" >> $PLOGFILE
fi
exit 0
